# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


MSC_PLANNING_API_ROUTE = 'pyams_msc.planning'
'''MSC planning API route'''


MSC_PLANNING_API_PATH = '/api/msc/planning'
'''MSC planning API default path'''


MSC_SESSION_API_ROUTE = 'pyams_msc.session'
'''MSC session API route'''


MSC_SESSION_API_PATH = '/api/msc/session/{session_id}'
'''MSC session API default path'''



# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


MSC_BOOKING_SEARCH_API_ROUTE = 'pyams_msc.booking.search'
'''MSC booking search API route'''


MSC_BOOKING_SEARCH_API_PATH = '/api/msc/booking'
'''MSC booking search API path'''


MSC_BOOKING_API_ROUTE = 'pyams_msc.booking'
'''MSC booking API route'''


MSC_BOOKING_API_PATH = '/api/msc/booking/{booking_id}'
'''MSC booking API path'''


MSC_BOOKING_VALIDATION_API_ROUTE = 'pyams_msc.booking.validation'
'''MSC booking validation API route'''


MSC_BOOKING_VALIDATION_API_PATH = '/api/msc/booking/{booking_id}/validate'
'''MSC booking validation API path'''



# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

from datetime import date, datetime, timedelta, timezone

from ZODB.POSException import POSKeyError
from ZODB.utils import p64
from cornice import Service
from cornice.validators import colander_validator
from hypatia.catalog import CatalogQuery
from hypatia.interfaces import ICatalog
from hypatia.query import And, Any, Eq, Ge, Lt, Or
from pyramid.httpexceptions import HTTPBadRequest, HTTPNotFound, HTTPOk, HTTPUnauthorized
from zope.intid.interfaces import IIntIds

from pyams_app_msc.feature.planning.api.interfaces import MSC_PLANNING_API_ROUTE, MSC_SESSION_API_ROUTE
from pyams_app_msc.feature.planning.api.schema import PlanningGetterResponse, SessionGetterResponse
from pyams_app_msc.feature.planning.interfaces import ISession
from pyams_app_msc.feature.profile.interfaces import IOperatorProfile
from pyams_app_msc.interfaces import VIEW_PLANNING_PERMISSION
from pyams_app_msc.shared.theater.interfaces import IMovieTheater
from pyams_catalog.query import CatalogResultSet
from pyams_content_api.feature.json.interfaces import IJSONExporter
from pyams_security.interfaces.base import USE_INTERNAL_API_PERMISSION
from pyams_security.rest import check_cors_origin, set_cors_headers
from pyams_utils.factory import get_interface_base_name
from pyams_utils.registry import get_utility
from pyams_utils.rest import STATUS, http_error, rest_responses
from pyams_utils.timezone import tztime

__docformat__ = 'restructuredtext'

from pyams_app_msc import _


planning_service = Service(name=MSC_PLANNING_API_ROUTE,
                           pyramid_route=MSC_PLANNING_API_ROUTE,
                           description="Planning getter service")


@planning_service.options(validators=(check_cors_origin, set_cors_headers))
def planning_options(request):
    """Planning service options"""
    return ''


planning_get_responses = rest_responses.copy()
planning_get_responses[HTTPOk.code] = PlanningGetterResponse(description="Planning getter response")


@planning_service.get(validators=(check_cors_origin, colander_validator, set_cors_headers),
                      response_schemas=planning_get_responses)
def get_planning(request):
    """Get planning information"""
    if not request.has_permission(USE_INTERNAL_API_PERMISSION, context=request.context):
        return http_error(request, HTTPUnauthorized)
    registry = request.registry
    profile = IOperatorProfile(request)
    catalog = get_utility(ICatalog)
    intids = get_utility(IIntIds)
    principal_id = request.principal.id
    params = And(Eq(catalog['object_types'], get_interface_base_name(IMovieTheater)),
                 Or(Eq(catalog['role:msc:manager'], principal_id),
                    Eq(catalog['role:msc:operator'], principal_id),
                    Eq(catalog['role:msc:contributor'], principal_id),
                    Eq(catalog['role:msc:reader'], principal_id)))
    sessions = []
    theaters = list(CatalogResultSet(CatalogQuery(catalog).query(params)))
    if not theaters:
        translate = request.localizer.translate
        return {
            'status': STATUS.SUCCESS.value,
            'message': translate(_("You are not assigned to any theater")),
            'results': sessions
        }
    append_session = sessions.append
    now = tztime(datetime.now(timezone.utc))
    today_start = tztime(datetime.combine(date.today(), datetime.min.time()))
    today_end = today_start + timedelta(days=profile.today_program_length)
    params = And(Eq(catalog['object_types'], get_interface_base_name(ISession)),
                 Any(catalog['parents'], [intids.register(theater) for theater in theaters]),
                 Lt(catalog['planning_start_date'], today_end),
                 Ge(catalog['planning_end_date'], now))
    for session in CatalogResultSet(CatalogQuery(catalog).query(params,
                                                                sort_index='planning_start_date')):
        exporter = registry.queryMultiAdapter((session, request), IJSONExporter,
                                              name='msc_mobile_api')
        if exporter is not None:
            append_session(exporter.to_json())
    return {
        'status': STATUS.SUCCESS.value,
        'results': sessions
    }
    
    
session_service = Service(name=MSC_SESSION_API_ROUTE,
                          pyramid_route=MSC_SESSION_API_ROUTE,
                          description="Session getter service")


@session_service.options(validators=(check_cors_origin, set_cors_headers))
def session_options(request):
    """Session service options"""
    return ''


session_get_responses = rest_responses.copy()
session_get_responses[HTTPOk.code] = SessionGetterResponse(description="Session getter response")


@session_service.get(permission=USE_INTERNAL_API_PERMISSION,
                     validators=(check_cors_origin, colander_validator, set_cors_headers),
                     response_schemas=session_get_responses)
def get_session(request):
    """Get session information"""
    if not request.has_permission(USE_INTERNAL_API_PERMISSION, context=request.context):
        return http_error(request, HTTPUnauthorized)
    session_id = request.matchdict['session_id']
    if not session_id:
        return http_error(request, HTTPBadRequest, "Missing session ID")
    try:
        session = request.context._p_jar.get(p64(int(session_id, 16)))
    except POSKeyError:
        return http_error(request, HTTPNotFound)
    if not ISession.providedBy(session):
        return http_error(request, HTTPBadRequest, "Bad session ID")
    if not request.has_permission(VIEW_PLANNING_PERMISSION, context=session):
        return {
            'status': STATUS.SUCCESS.value,
            'message': _("You are not allowed to access this session"),
            'session': None
        }
    exporter = request.registry.queryMultiAdapter((session, request), IJSONExporter,
                                                  name='msc_mobile_session_api')
    if exporter is not None:
        return {
            'status': STATUS.SUCCESS.value,
            'session': exporter.to_json()
    }
    return http_error(request, HTTPNotFound)



#
# Copyright (c) 2015-2023 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

"""PyAMS_*** module

"""

from pyams_app_msc.shared.theater.interfaces import IMovieTheater
from pyams_form.interfaces import INPUT_MODE
from pyams_form.template import widget_template_config
from pyams_form.widget import FieldWidget
from pyams_security_views.widget.principal import PrincipalWidget
from pyams_utils.traversing import get_parent
from pyams_utils.url import absolute_url
from pyams_zmi.interfaces import IAdminLayer

__docformat__ = 'restructuredtext'


@widget_template_config(mode=INPUT_MODE,
                        template='templates/principal-input.pt',
                        layer=IAdminLayer)
class PrincipalSelectWidget(PrincipalWidget):
    """Principal select widget"""

    @property
    def ajax_url(self):
        theater = get_parent(self.context, IMovieTheater)
        return absolute_url(theater, self.request, 'get-principals.json')


def PrincipalSelectFieldWidget(field, request):  # pylint: disable=invalid-name
    """Principal select field widget"""
    return FieldWidget(field, PrincipalSelectWidget(request))



# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

from colander import Boolean, Integer, MappingSchema, OneOf, SchemaNode, SequenceSchema, String, drop

from pyams_app_msc.feature.booking import BOOKING_STATUS
from pyams_utils.rest import BaseResponseSchema

__docformat__ = 'restructuredtext'


class BookingSearchQuery(MappingSchema):
    """Booking search query"""
    with_session_info = SchemaNode(Boolean(),
                                   description="Include session information",
                                   missing=False)


class BookingSearchRequest(MappingSchema):
    """Booking information request"""
    querystring = BookingSearchQuery()


class BookingInfo(MappingSchema):
    """Booking info"""
    id = SchemaNode(String(),
                    description="Booking ID")
    session_id = SchemaNode(String(),
                            description="Booking session ID")
    activity = SchemaNode(String(),
                          description="Booking activity name",
                          missing=drop)
    recipient = SchemaNode(String(),
                           description="Booking recipient name")
    establishment = SchemaNode(String(),
                               description="Booking establishment name")
    status = SchemaNode(String(),
                        description="Booking status")
    nb_participants = SchemaNode(Integer(),
                                 description="Booking participants count")
    participants_age = SchemaNode(String(),
                                  description="Booking participants age")
    nb_accompanists = SchemaNode(Integer(),
                                 description="Booking accompanists count")
    accompanying_ratio = SchemaNode(Integer(),
                                    description="Accompanying ratio defines the number of participants for "
                                                "which an accompanying person will have free access")
    nb_free_accompanists = SchemaNode(Integer(),
                                      description="Booking free accompanists count")
    nb_groups = SchemaNode(Integer(),
                           description="Booking groups count")
    price = SchemaNode(String(),
                       description="Booking price")
    cultural_pass = SchemaNode(Boolean(),
                               description="Booking cultural pass marker")
    comments = SchemaNode(String(),
                          description="Booking comments",
                          missing=drop)
    notepad = SchemaNode(String(),
                         description="Booking notepad",
                         missing=drop)
    can_update = SchemaNode(Boolean(),
                            description="Booking update permission flag",
                            missing=False)


class BookingList(SequenceSchema):
    """Booking list"""
    booking = BookingInfo()


class BookingSearchResult(BaseResponseSchema):
    """Booking search result"""
    results = BookingList(description="Bookings list",
                          missing=drop)

    
class BookingSearchResponse(MappingSchema):
    """Booking search response"""
    body = BookingSearchResult()
    

class BookingInfoQuery(MappingSchema):
    """Booking information query"""
    with_session_info = SchemaNode(Boolean(),
                                   description="Include session information",
                                   missing=False)
    
    
class BookingInfoRequest(MappingSchema):
    """Booking information request"""
    querystring = BookingInfoQuery()
    
    
class FullBookingInfo(BookingInfo):
    """Full booking info"""


class FullBookingInfoResult(BaseResponseSchema):
    """Full booking info result"""
    booking = FullBookingInfo(description="Full booking info",
                                missing=drop)
    
    
class FullBookingInfoResponse(MappingSchema):
    """Full booking info response"""
    body = FullBookingInfoResult()


class PriceItem(MappingSchema):
    """Price item"""
    id = SchemaNode(String(),
                    description="Price ID")
    name = SchemaNode(String(),
                      description="Price name")
    
    
class PriceList(SequenceSchema):
    """Price list"""
    price = PriceItem()
    

class CommentsList(SequenceSchema):
    """Comments list"""
    comment = SchemaNode(String(),
                         description="Comment")

    
class BookingValidationInfo(FullBookingInfo):
    """Booking validation info"""
    price = SchemaNode(String(),
                       description="Booking price",
                       missing=drop)
    available_prices = PriceList(description="Available prices",
                                 missing=drop)
    notify_subject = SchemaNode(String(),
                                description="Notification email subject",
                                missing=drop)
    notify_message = SchemaNode(String(),
                                description="Notification email message",
                                missing=drop)
    can_send_reminder = SchemaNode(Boolean(),
                                   description="Boolean flag used to specify if a reminder "
                                               "can be sentto recipient")
    reminder_subject = SchemaNode(String(),
                                  description="Reminder email subject",
                                  missing=drop)
    reminder_message = SchemaNode(String(),
                                  description="Reminder email message",
                                  missing=drop)
    notepads = CommentsList(desription="Notepads",
                            missing=drop)
    
    
class BookingValidationRequest(BookingInfoRequest):
    """Booking validation request"""

    
class BookingValidationGetterResult(BaseResponseSchema):
    """Booking validation getter result"""
    booking = BookingValidationInfo()
    
    
class BookingValidationGetterResponse(MappingSchema):
    """Booking validation getter response"""
    body = BookingValidationGetterResult()
    
    
class BookingValidationPostInfo(MappingSchema):
    """Booking validation post info"""
    status = SchemaNode(String(),
                        description="Booking status",
                        validator=OneOf((
                            BOOKING_STATUS.OPTION.value,
                            BOOKING_STATUS.ACCEPTED.value
                        )))
    price = SchemaNode(String(),
                       description="Selected booking price")
    accompanying_ratio = SchemaNode(Integer(),
                                    description="Accompanying ratio defines the number of participants for "
                                                "which an accompanying person will have free access",
                                    missing=0)
    notify_recipient = SchemaNode(Boolean(),
                                  description="Recipient notification flag",
                                  missing=False)
    notify_subject = SchemaNode(String(),
                                description="Recipient notification subject",
                                missing=drop)
    notify_message = SchemaNode(String(),
                                description="Recipient notification message",
                                missing=drop)
    include_quotation = SchemaNode(Boolean(),
                                   description="Include quotation flag",
                                   missing=False)
    quotation_message = SchemaNode(String(),
                                   description="Message added to quotation",
                                   missing=drop)
    send_reminder = SchemaNode(Boolean(),
                               description="Reminder notification flag",
                               missing=False)
    reminder_subject = SchemaNode(String(),
                                  description="Reminder subject",
                                  missing=drop)
    reminder_message = SchemaNode(String(),
                                  description="Reminder message",
                                  missing=drop)
    notepad = SchemaNode(String(),
                         description="Booking notepad",
                         missing=drop)
    
    
class BookingValidationPostRequest(MappingSchema):
    """Booking validation post request"""
    body = BookingValidationPostInfo()
    
    
class BookingValidationPostResult(BaseResponseSchema):
    """Booking validation post result"""
    
    
class BookingValidationPostResponse(MappingSchema):
    """Booking validation post response"""
    body = BookingValidationPostResult()
    